/* falkoveto_robot

   Falkövető robot, amely a falat és az akadályokat két TCRT5000
   szenzorral érzékeli.

   A program a baloldali szenzor jelzései alapján követi a falat, a
   szenzor által érzékelt értéket az L_LIMIT és L_LIMIT_MIN között
   tartva. Mindeközben az elülső (front) szenzor a kikerülendő
   akadályokat észleli. Akadály észlelése eseténa robot jobbra fordul,
   amíg az elülső szenzor jele F_LIMIT_MIN alá nem csökken.

   Tested with Arduino IDE 1,8,13
*/

#define B1A 7          // Bal motor előre
#define B1B 6          // Bal motor hátra
#define A1A 5          // Jobb motor előre
#define A1B 4          // Jobb motor hátra

#define L_sense A0
#define L_LED   10
#define F_sense A1
#define F_LED    9
#define NMEAS   10

#define L_LIMIT_LOW  50
#define L_LIMIT      80
#define F_LIMIT_LOW  30
#define F_LIMIT      50


uint32_t meres(int p_sense, int p_LED) {
  uint32_t sum0, sum1;
  digitalWrite(p_LED, LOW);
  sum0 = 0;
  for (int i = 0; i < NMEAS; i++) {
    sum0 += analogRead(p_sense);
  }
  digitalWrite(p_LED, HIGH);
  sum1 = 0;
  for (int i = 0; i < NMEAS; i++) {
    sum1 += analogRead(p_sense);
  }
  if (sum0 > sum1) sum0 = sum1;
  return ((sum1 - sum0) / NMEAS);
}

void setup() {
  analogReference(DEFAULT);
  pinMode(L_LED, OUTPUT);
  pinMode(F_LED, OUTPUT);
  // Motorvezérlő lábak konfigurálása
  pinMode(B1A, OUTPUT);
  pinMode(B1B, OUTPUT);
  pinMode(A1A, OUTPUT);
  pinMode(A1B, OUTPUT);
  digitalWrite(B1A, LOW);
  digitalWrite(B1B, LOW);
  digitalWrite(A1A, LOW);
  digitalWrite(A1B, LOW);
}

void loop() {
  motor_forward();
  uint32_t bal = meres(L_sense, L_LED);
  uint32_t kozep = meres(F_sense, F_LED);
  //  uint32_t jobb = meres(R_sense, R_LED);

  if (bal > L_LIMIT) {
    motor_right();
  }
  else if (bal < L_LIMIT_LOW) {
    motor_left();
  }
  else {
    motor_forward();
  }

  if (kozep > F_LIMIT) {
    while (kozep > F_LIMIT_LOW) {
      kozep = meres(F_sense, F_LED);
      motor_right();
    }
  } else { delay(10); }
}

void motor_forward() {
  digitalWrite(B1A, HIGH);
  digitalWrite(B1B, LOW);
  digitalWrite(A1A, HIGH);
  digitalWrite(A1B, LOW);
}

void motor_reverse() {
  digitalWrite(B1A, LOW);
  digitalWrite(B1B, HIGH);
  digitalWrite(A1A, LOW);
  digitalWrite(A1B, HIGH);
}

void motor_left() {
  digitalWrite(B1A, LOW);
  digitalWrite(B1B, HIGH);
  digitalWrite(A1A, HIGH);
  digitalWrite(A1B, LOW);
}

void motor_right() {
  digitalWrite(B1A, HIGH);
  digitalWrite(B1B, LOW);
  digitalWrite(A1A, LOW);
  digitalWrite(A1B, HIGH);
}

void motor_stop() {
  digitalWrite(B1A, LOW);
  digitalWrite(B1B, LOW);
  digitalWrite(A1A, LOW);
  digitalWrite(A1B, LOW);
}
